#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glaux.h>
#include "PMViewerApp.h"
#include "resource.h"
#include "View.h"
#include "pmmodel.h"
#include "viewdialog.h"

using namespace Windows;


CView::CView ()
{
	ZeroMemory(&m_settings, sizeof(CViewSettings));
	ZeroMemory(&m_keysHeld, sizeof(m_keysHeld));
	ZeroMemory(&m_keysPressed, sizeof(m_keysPressed));

	m_pmm = NULL;
	m_pmw = NULL;
	

	/*m_settings.lightAmbient[0] = 0.5f;
	m_settings.lightAmbient[1] = 0.5f;
	m_settings.lightAmbient[2] = 0.5f;
	m_settings.lightAmbient[3] = 1.0f;

	m_settings.lightDiffuse[0] = 1.0f;
	m_settings.lightDiffuse[1] = 1.0f;
	m_settings.lightDiffuse[2] = 1.0f;
	m_settings.lightDiffuse[3] = 1.0f;

	m_settings.lightPossition[0] = -1.0f;
	m_settings.lightPossition[1] = 1.0f;
	m_settings.lightPossition[2] = 1.0f;
	m_settings.lightPossition[3] = 0.0f;*/

	m_settings.showAxes = 1;
	m_settings.showText = 1;
	m_settings.showLighting = 1;
	m_settings.showColor = 1;
	m_settings.showTextures = 1;
	m_settings.showCullPolygons = 1;
	ResetPosition();

	SetTimer(m_hWnd, IDT_TIMER, 20, CView::TimerLoop);
}

CView::~CView ()
{
	if (m_pmm != NULL)
		delete m_pmm;
	if (m_pmw != NULL)
		delete m_pmw;

	KillTimer(m_hWnd, IDT_TIMER);
}

void CView::TimerLoop (HWND hWnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime)
{
	((CPMViewerApp*)GetApp())->EventLoop();
}

HWND CView::Create (HWND hWndParent /*= NULL*/)
{
	GLuint		pixelFormat;
	int			bits = 16;
	int			width = 640;
	int			height = 480;

	// Create Window Class
	CWinClass wc(TEXT("View"));

	// Get WNDCLASSEX structure
	WNDCLASSEX wcx = wc.GetWndClassEx();
	wcx.style = wcx.style | CS_HREDRAW | CS_VREDRAW | CS_OWNDC;

	// Store Structure
	wc.SetWndClassEx(wcx);

	// Register Window Class
	wc.RegisterClassEx();

	// Sunken Edge
	DWORD dwExStyle = WS_EX_CLIENTEDGE;

	// Create View Window
	HWND hWnd = CreateEx(dwExStyle, TEXT("View"), TEXT(""),
		WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, 0, 0, 0, 0, hWndParent, NULL);

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		bits,
		0, 0, 0, 0, 0, 0,
		0,
		0,
		0,
		0, 0, 0, 0,
		16,
		0,
		0,
		PFD_MAIN_PLANE,
		0,
		0, 0, 0
	};

	if (!(m_hDC = GetDC(hWnd))) {
		KillGLWindow();
		MessageBox(NULL, "Could not create a GL Device Context.","ERROR",MB_OK | MB_ICONERROR);
		return NULL;
	}

	if (!(pixelFormat = ChoosePixelFormat(m_hDC, &pfd))) {
		KillGLWindow();
		MessageBox(NULL, "Could not find a suitable pixel format.","ERROR",MB_OK | MB_ICONERROR);
		return NULL;
	}

	if (!SetPixelFormat(m_hDC, pixelFormat, &pfd)) {
		KillGLWindow();
		MessageBox(NULL, "Can't set the pixel format.","ERROR",MB_OK | MB_ICONERROR);
		return NULL;
	}

	if (!(m_hRC = wglCreateContext(m_hDC))) {
		KillGLWindow();
		MessageBox(NULL, "Can't create a GL Rendering Context.","ERROR",MB_OK | MB_ICONERROR);
		return NULL;
	}

	if (!wglMakeCurrent(m_hDC, m_hRC)) {
		KillGLWindow();
		MessageBox(NULL, "Can't activate the GL Rendering Context.","ERROR",MB_OK | MB_ICONERROR);
		return NULL;
	}

	ShowWindow(hWnd, SW_SHOW);
	SetForegroundWindow(hWnd);
	SetFocus(hWnd);
	ReSizeGLScene(width, height);

	if (!InitGL()) {
		KillGLWindow();
		MessageBox(NULL, "Initialization Failed.","ERROR",MB_OK | MB_ICONERROR);
		return NULL;
	}

	return hWnd;
}

LRESULT CView::WndProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_CLOSE:
            DestroyWindow(hWnd);
        break;
	case WM_SIZE:
		ReSizeGLScene(LOWORD(lParam), HIWORD(lParam));
		return 0;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case ID_FILE_OPEN:
			{
				string filename = OpenFileDlg(m_hWnd);
				if (filename.size() > 0)
				{
					if (m_pmm != NULL) {
						delete m_pmm;
						m_pmm = NULL;
					}
					if (m_pmw != NULL) {
						delete m_pmw;
						m_pmw = NULL;
					}
					
					switch (GetFileType(filename))
					{
					case 0:
						m_settings.file_loaded = 0;
						break;
					case 2:
						m_pmm = new CPMModelGL();
						m_pmm->LoadFile(filename);
						m_settings.file_loaded = 2;
						break;
					case 3:
						m_pmw = new CPMWorld();
						m_pmw->LoadFile(filename);
						m_settings.file_loaded = 3;
						break;
					}
				}
				::SendMessage(m_hWndParent, TB_CHECKBUTTON, (WPARAM)ID_CONTROL_PAN, (LPARAM) MAKELONG(TRUE,0 ));
				::SendMessage(m_hWndParent, TB_CHECKBUTTON, (WPARAM)ID_DISPLAY_SHOWAXES, (LPARAM) MAKELONG(TRUE,0 ));
				::SendMessage(m_hWndParent, TB_CHECKBUTTON, (WPARAM)ID_DISPLAY_COLOR, (LPARAM) MAKELONG(TRUE,0 ));
				::SendMessage(m_hWndParent, TB_CHECKBUTTON, (WPARAM)ID_DISPLAY_TEXTURES, (LPARAM) MAKELONG(TRUE,0 ));
				::SendMessage(m_hWndParent, TB_CHECKBUTTON, (WPARAM)ID_DISPLAY_CULLPOLYGONS, (LPARAM) MAKELONG(TRUE,0 ));
			}
			return 0;
		case ID_CONTROL_PAN:
			m_settings.controlState = 0;
			return 0;
		case ID_CONTROL_ROTATE:
			m_settings.controlState = 1;
			return 0;
		case ID_CONTROL_ZOOM:
			m_settings.controlState = 2;
			return 0;
		case ID_DISPLAY_SHOWAXES:
			toggle(m_settings.showAxes);
			return 0;
		case ID_DISPLAY_TEXTURES:
			toggle(m_settings.showTextures);
			return 0;
		case ID_DISPLAY_BOUNDINGBOX:
			toggle(m_settings.showBoundingBox);
			return 0;
		case ID_UTILITY_DATAVIEW:
			CPMDataDialog DataDialog(IDD_DIALOG2, m_pmm, m_hWnd);
			DataDialog.DoModal();
			return 0;
		}
		break;
	case WM_KEYDOWN:
		{
			m_keysHeld[wParam] = TRUE;
			m_keysPressed[wParam] = TRUE;
		}
		break;
	case WM_KEYUP:
		{
			m_keysHeld[wParam] = FALSE;
			m_keysPressed[wParam] = FALSE;
		}
		break;
	}

	return CWin::WndProc(hWnd, uMsg, wParam, lParam);
}

void CView::EventLoop ()
{
	DrawGLScene();
	SwapBuffers(m_hDC);
	KeyControl();
}

void CView::KeyControl ()
{
	KeyControlState();
}

void CView::KeyControlState ()
{
	switch (m_settings.controlState)
	{
	case 0:
		if (m_keysHeld[VK_LEFT])
			m_settings.x -= (m_keysHeld[VK_SHIFT]) ? 2.5f : 0.5f;
		if (m_keysHeld[VK_RIGHT])
			m_settings.x += (m_keysHeld[VK_SHIFT]) ? 2.5f : 0.5f;
		if (m_keysHeld[VK_UP])
			m_settings.y += (m_keysHeld[VK_SHIFT]) ? 2.5f : 0.5f;
		if (m_keysHeld[VK_DOWN])
			m_settings.y -= (m_keysHeld[VK_SHIFT]) ? 2.5f : 0.5f;
		break;
	case 1:
		if (m_keysHeld[VK_LEFT])
			m_settings.yrot -= (m_keysHeld[VK_SHIFT]) ? 5.0f : 1.0f;
		if (m_keysHeld[VK_RIGHT])
			m_settings.yrot += (m_keysHeld[VK_SHIFT]) ? 5.0f : 1.0f;
		if (m_keysHeld[VK_UP])
			m_settings.xrot -= (m_keysHeld[VK_SHIFT]) ? 5.0f : 1.0f;
		if (m_keysHeld[VK_DOWN])
			m_settings.xrot += (m_keysHeld[VK_SHIFT]) ? 5.0f : 1.0f;
		break;
	case 2:
		if (m_keysHeld[VK_UP])
			m_settings.z -= (m_keysHeld[VK_SHIFT]) ? 5.0f : 1.0f;
		if (m_keysHeld[VK_DOWN])
			m_settings.z += (m_keysHeld[VK_SHIFT]) ? 5.0f : 1.0f;
		break;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////
// DRAWING
//////////////////////////////////////////////////////////////////////////////////////////

void CView::DrawAxes () {
	
	if (!m_settings.showAxes)
		return;

	glDisable(GL_TEXTURE_2D);
	glBegin(GL_LINES);
	glColor3f(1.0f, 0.0f, 0.0f);
	glVertex3f(-20.0f, 0.0f, 0.0f);
	glVertex3f(20.0f, 0.0f, 0.0f);
	glColor3f(0.0f, 1.0f, 0.0f);
	glVertex3f(0.0f, 20.0f, 0.0f);
	glVertex3f(0.0f, -20.0f, 0.0f);
	glColor3f(0.0f, 0.0f, 1.0f);
	glVertex3f(0.0f, 0.0f, 20.0f);
	glVertex3f(0.0f, 0.0f, -20.0f);
	glEnd();
}

void CView::MoveWorld ()
{
	// Translate
	glTranslatef(m_settings.x, m_settings.y, m_settings.z);

	// Rotate
	glRotatef(m_settings.xrot, 1.0f, 0.0f, 0.0f);
	glRotatef(m_settings.yrot, 0.0f, 1.0f, 0.0f);
}

void CView::ResetPosition ()
{
	m_settings.x = 0.0f;
	m_settings.y = -2.0f;
	m_settings.z = -55.0f;
	m_settings.xrot = 0.0f;
	m_settings.yrot = 0.0f;
	m_settings.xspeed = 0.02f;
	m_settings.yspeed = 0.0f;
}

// type 0=Error, 1=TPL, 2=Model, 3=World/Other
int CView::GetFileType (string filename)
{
	FILE *fh = fopen(filename.c_str(), "rb");
	if (fh != NULL) {
		u32 type;
		fread(&type, sizeof(u32), 1, fh);
		byteSwapU32(type);
		if (type == 0x0020AF30)
			return 1;

		fseek(fh, 0x14C, SEEK_SET);
		fread(&type, sizeof(u32), 1, fh);
		byteSwapU32(type);
		if (type == 0x1B0)
			return 2;
        
		fclose(fh);
		return 3;
	}

	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
// OPENGL
//////////////////////////////////////////////////////////////////////////////////////////

GLvoid	CView::ReSizeGLScene (GLsizei width, GLsizei height) {
	if (height == 0)
		height = 1;

	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLfloat)width  / (GLfloat)height, 0.1f, 600.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

int CView::InitGL (GLvoid) {

	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);

	glDepthFunc(GL_LEQUAL);
	//glAlphaFunc(GL_GREATER, 0.95f);
	//glAlphaFunc(GL_EQUAL, 1.0f);
	//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	//glEnable(GL_BLEND);

	glColor4f(1.0f, 1.0f, 1.0f, 0.5f);
	
	//glEnable(GL_COLOR_MATERIAL);
	//glLightfv(GL_LIGHT1, GL_AMBIENT, m_settings.lightAmbient);
	//glLightfv(GL_LIGHT1, GL_DIFFUSE, m_settings.lightDiffuse);
	//glLightfv(GL_LIGHT1, GL_POSITION, m_settings.lightPossition);
	//glEnable(GL_LIGHT1);
	//glEnable(GL_LIGHTING);

	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	//glFont = new FontBuilder(hDC);
	//settings.base = glBuildFont(hDC);

	return TRUE;
}

int CView::DrawGLScene (GLvoid) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	MoveWorld();

	glColor3f(1.0f, 1.0f, 1.0f);
	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	glEnable(GL_TEXTURE_2D);
	//glEnable(GL_ALPHA_TEST);

	if (m_settings.file_loaded == 2) {
		m_pmm->showTextures(m_settings.showTextures);
		m_pmm->TransformNode(m_pmm->m_Block25.size()-1);
		if (m_settings.showBoundingBox)
			m_pmm->DrawBoundingBox();
	} else if (m_settings.file_loaded == 3)	{
		m_pmw->DrawWorld();
	}

	if (m_settings.showAxes)
		DrawAxes();

	return TRUE;
}

GLvoid CView::KillGLWindow (GLvoid) {

	if (m_hRC) {
		if (!wglMakeCurrent(NULL, NULL))
			MessageBox(NULL, "Release of DC and RC failed.","Shutdown Error",MB_OK | MB_ICONINFORMATION);
		if (!wglDeleteContext(m_hRC))
			MessageBox(NULL, "Release Rendering Context Failed.","Shutdown Error",MB_OK | MB_ICONINFORMATION);
		m_hRC = NULL;
	}

	if (m_hDC && !ReleaseDC(m_hWnd, m_hDC)) {
		MessageBox(NULL, "Release Device Context Failed.","Shutdown Error",MB_OK | MB_ICONINFORMATION);
		m_hDC = NULL;
	}

//	KillTimer(hWnd, ID_TIMER);
//	glFont->KillFont();
}

