#ifndef VIEW_H
#define VIEW_H

#include <windows.h>
#include <vector>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glaux.h>
#include "Windows\Wincore.h"
#include "pmmodelgl.h"
#include "pmworld.h"
#include "common.h"

using std::vector;
using namespace Windows;

class CView : public CWin
{
public:
	CView ();
	virtual ~CView ();
	HWND Create (HWND hWndParent /*= NULL*/);
	HWND GetHWnd () { return m_hWnd; }

	static void CALLBACK TimerLoop (HWND hWnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime);
	virtual void EventLoop ();
	void KeyControl ();
	void KeyControlState ();
	int GetFileType (string filename);

	// OpenGL
	GLvoid	ReSizeGLScene (GLsizei width, GLsizei height);
	int InitGL (GLvoid);
	int DrawGLScene (GLvoid);
	GLvoid KillGLWindow (GLvoid);

private:
	virtual LRESULT WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	void DrawAxes();
	void MoveWorld();
	void ResetPosition();

	HGLRC		m_hRC;
	HDC			m_hDC;
	bool		m_keysHeld[256];
	bool		m_keysPressed[256];

	struct CViewSettings {
		int		file_loaded;
		int		controlState;
		int		showAxes;
		int		showText;
		int		showLighting;
		int		showColor;
		int		showTextures;
		int		showCullPolygons;
		int		showBoundingBox;
		float	x;
		float	y;
		float	z;
		float	xrot;
		float	yrot;
		float	xspeed;
		float	yspeed;
		//GLfloat* lightAmbient[];
		//GLfloat	lightDiffuse[4];
		//GLfloat	lightPossition[4];
	};

	CPMModelGL*		m_pmm;
	CPMWorld*		m_pmw;
	CViewSettings	m_settings;
};

#endif // VIEW_H