#include "color.h"
#include "common.h"

void Color::RGB565_Unpack(u16 c)
{
	a = 255;
	r = ((c >> 11)&31) << 3;
	r += r >> 5;
	g = ((c >> 5)&63) << 2;
	g += g >> 6;
	b = ((c >> 0)&31) << 3;
	b += b >> 5;
}

void Color::I4_Unpack(u8 c)
{

	a = 255;
	r = ((c & 0x0F) << 4) + (c & 0x0F);
	g = r;
	b = r;
}

void Color::I8_Unpack(u8 c)
{
	a = 255;
	r = c;
	g = c;
	b = c;
}

void Color::IA4_Unpack(u8 c)
{
	a = (c & 0xF0) + ((c & 0xF0) >> 4);
	r = ((c & 0x0F) << 4) + (c & 0x0F);
	g = r;
	b = r;
}

void Color::IA8_Unpack(u16 c)
{
	a = c >> 8;
	r = c & 0xFF;
	g = c & 0xFF;
	b = c & 0xFF;
}

void Color::RGB5A3_Unpack(u16 c)
{
	if ((c >> 15)&1 == 1)
		RGB4A3_Unpack(c);
	else
		RGB5_Unpack(c);
}

void Color::RGB4A3_Unpack(u16 c)
{
	a = ((c >> 12) & 7) << 5;
	a += a >> 3;
	r = ((c >> 8) & 15) << 4;
	r += r >> 4;
	g = ((c >> 4) & 15) << 4;
	g += g >> 4;
	b = ((c >> 0) & 15) << 4;
	b += b >> 4;
}

void Color::RGB5_Unpack(u16 c)
{
	a = 255;
	r = ((c >> 11) & 31) << 3;
	r += r >> 5;
	g = ((c >> 6) & 31) << 3;
	g += g >> 5;
	b = ((c >> 1) & 31) << 3;
	b += b >> 5;
}

void Color::RGBA8_Unpack(u16 c0, u16 c1)
{
	a = (c0 & 0xFF00) >> 8;
	r = (c0 & 0xFF);
	g = (c1 & 0xFF00) >> 8;
	b = (c1 & 0xFF);
}

u32 Color::RGBA8_Get()
{
	return (r<<24) | (g<<16) | (b<<8) | a;
}

void Color::Blend(Color c0, Color c1, u8 ratio)
{
	u8 a1 = ratio;
	u8 a2 = 256-ratio;
	r = (a2*c0.r + a1*c1.r) >> 8;
	g = (a2*c0.g + a1*c1.g) >> 8;
	b = (a2*c0.b + a1*c1.b) >> 8;
	a = (a2*c0.a + a1*c1.a) >> 8;
}

void Color::Set(u8 cr, u8 cg, u8 cb, u8 ca)
{
	r = cr;
	g = cg;
	b = cb;
	a = ca;
}