#include "pmmodel.h"

using namespace std;

CPMModel::CPMModel ()
{
	ZeroMemory(&m_header, sizeof(PMMHeader));
}

CPMModel::~CPMModel ()
{
}

bool CPMModel::LoadFile (const string &filename)
{
	m_filename = filename;
	
	FILE* fh;
	fh = fopen(m_filename.c_str(), "rb");
	if (fh == NULL)
		return false;

	ReadHeader(fh);
	ReadBlock2(fh);
	ReadBlock3(fh);
	ReadBlock4(fh);
	ReadBlock5(fh);
	ReadBlock6(fh);
	ReadBlock7(fh);
	ReadBlock8(fh);
	ReadBlock9(fh);
	ReadBlock10(fh);
	ReadBlock18(fh);
	ReadBlock19(fh);
	ReadBlock20(fh);
	ReadBlock21(fh);
	ReadBlock22(fh);
	ReadBlock23(fh);
	ReadBlock24(fh);
	ReadBlock25(fh);
	ReadBlock26(fh);
	ReadBlock27(fh);
	
	fclose(fh);

	return true;
}

void CPMModel::ReadHeader (FILE* fh)
{
	fread(&m_header, sizeof(PMMHeader), 1, fh);

	byteSwapU32(m_header.addrBlock27);
	byteSwapU32(m_header.u0xC4);
	byteSwapU32(m_header.u0xC8);
	byteSwapU32(m_header.u0xCC);
	byteSwapF32(m_header.u0xD0);
	byteSwapF32(m_header.u0xD4);
	byteSwapF32(m_header.u0xD8);
	byteSwapF32(m_header.u0xDC);
	byteSwapF32(m_header.u0xE0);
	byteSwapF32(m_header.u0xE4);

	for (int x=0; x<25; x++)
		byteSwapU32(m_header.numBlocks[x]);
	for (int x=0; x<25; x++)
		byteSwapU32(m_header.blockOffset[x]);
}

void CPMModel::ReadBlock2 (FILE* fh)
{
	m_Block2.resize(m_header.numBlocks[BLOCK2]);

	fseek(fh, m_header.blockOffset[BLOCK2], SEEK_SET);
	fread(&m_Block2[0], sizeof(PMMBlock2), m_header.numBlocks[BLOCK2], fh);

	for (int x=0; x<m_Block2.size(); x++)
	{
		byteSwapS32(m_Block2[x].idxBlock4);
		byteSwapS32(m_Block2[x].numBlock4);
		byteSwapU32(m_Block2[x].u0x48);
		byteSwapU32(m_Block2[x].u0x4C);
		byteSwapU32(m_Block2[x].u0x50);
		byteSwapU32(m_Block2[x].u0x54);
		byteSwapS32(m_Block2[x].idxBlock18);
		byteSwapS32(m_Block2[x].numBlock18);
		byteSwapS32(m_Block2[x].endVertex);
		byteSwapU32(m_Block2[x].u0x64);
		byteSwapU32(m_Block2[x].u0x68);
		byteSwapU32(m_Block2[x].u0x6C);
		byteSwapU32(m_Block2[x].u0x70);
		byteSwapU32(m_Block2[x].u0x74);
		byteSwapU32(m_Block2[x].u0x78);
		byteSwapU32(m_Block2[x].u0x7C);
		byteSwapU32(m_Block2[x].u0x80);
		byteSwapU32(m_Block2[x].u0x84);
		byteSwapU32(m_Block2[x].u0x88);
		byteSwapU32(m_Block2[x].u0x8C);
		byteSwapU32(m_Block2[x].u0x90);
		byteSwapU32(m_Block2[x].u0x94);
		byteSwapS32(m_Block2[x].idxBlock22);
		byteSwapS32(m_Block2[x].numBlock22);
		byteSwapU32(m_Block2[x].u0xA0);
		byteSwapU32(m_Block2[x].u0xA4);
	}
}

void CPMModel::ReadBlock3 (FILE* fh)
{
	m_Block3.resize(m_header.numBlocks[BLOCK3]);

	fseek(fh, m_header.blockOffset[BLOCK3], SEEK_SET);
	fread(&m_Block3[0], sizeof(PMMBlock3), m_header.numBlocks[BLOCK3], fh);

	for (int x=0; x<m_Block3.size(); x++)
	{
		byteSwapU32(m_Block3[x].extention);
		byteSwapU32(m_Block3[x].vertices);
	}
}

void CPMModel::ReadBlock4 (FILE* fh)
{
	m_Block4.resize(m_header.numBlocks[BLOCK4]);

	fseek(fh, m_header.blockOffset[BLOCK4], SEEK_SET);
	fread(&m_Block4[0], sizeof(PMMBlock4), m_header.numBlocks[BLOCK4], fh);

	for (int x=0; x<m_Block4.size(); x++)
	{
		byteSwapF32(m_Block4[x].x);
		byteSwapF32(m_Block4[x].y);
		byteSwapF32(m_Block4[x].z);
	}
}

void CPMModel::ReadBlock5 (FILE* fh)
{
	m_Block5.resize(m_header.numBlocks[BLOCK5]);

	fseek(fh, m_header.blockOffset[BLOCK5], SEEK_SET);
	fread(&m_Block5[0], sizeof(PMMBlock5), m_header.numBlocks[BLOCK5], fh);

	for (int x=0; x<m_Block5.size(); x++)
		byteSwapU32(m_Block5[x].vertex);
}

void CPMModel::ReadBlock6 (FILE* fh)
{
	m_Block6.resize(m_header.numBlocks[BLOCK6]);

	fseek(fh, m_header.blockOffset[BLOCK6], SEEK_SET);
	fread(&m_Block6[0], sizeof(PMMBlock6), m_header.numBlocks[BLOCK6], fh);

	for (int x=0; x<m_Block6.size(); x++)
	{
		byteSwapF32(m_Block6[x].nx);
		byteSwapF32(m_Block6[x].ny);
		byteSwapF32(m_Block6[x].nz);
	}
}

void CPMModel::ReadBlock7 (FILE* fh)
{
	m_Block7.resize(m_header.numBlocks[BLOCK7]);

	fseek(fh, m_header.blockOffset[BLOCK7], SEEK_SET);
	fread(&m_Block7[0], sizeof(PMMBlock7), m_header.numBlocks[BLOCK7], fh);

	for (int x=0; x<m_Block7.size(); x++)
		byteSwapU32(m_Block7[x].idxBlock6);
}

void CPMModel::ReadBlock8 (FILE* fh)
{
	m_Block8.resize(m_header.numBlocks[BLOCK8]);

	fseek(fh, m_header.blockOffset[BLOCK8], SEEK_SET);
	fread(&m_Block8[0], sizeof(PMMBlock8), m_header.numBlocks[BLOCK8], fh);
}

void CPMModel::ReadBlock9 (FILE* fh)
{
	m_Block9.resize(m_header.numBlocks[BLOCK9]);

	fseek(fh, m_header.blockOffset[BLOCK9], SEEK_SET);
	fread(&m_Block9[0], sizeof(PMMBlock9), m_header.numBlocks[BLOCK9], fh);

	for (int x=0; x<m_Block9.size(); x++)
		byteSwapU32(m_Block9[x].idxBlock8);
}

void CPMModel::ReadBlock10 (FILE* fh)
{
	m_Block10.resize(m_header.numBlocks[BLOCK10]);

	fseek(fh, m_header.blockOffset[BLOCK10], SEEK_SET);
	fread(&m_Block10[0], sizeof(PMMBlock10), m_header.numBlocks[BLOCK10], fh);

	for (int x=0; x<m_Block10.size(); x++)
		byteSwapU32(m_Block10[x].vertex);
}

void CPMModel::ReadBlock18 (FILE* fh)
{
	m_Block18.resize(m_header.numBlocks[BLOCK18]);

	fseek(fh, m_header.blockOffset[BLOCK18], SEEK_SET);
	fread(&m_Block18[0], sizeof(PMMBlock18), m_header.numBlocks[BLOCK18], fh);

	for (int x=0; x<m_Block18.size(); x++)
	{
		byteSwapF32(m_Block18[x].s);
		byteSwapF32(m_Block18[x].t);
	}
}

void CPMModel::ReadBlock19 (FILE* fh)
{
	m_Block19.resize(m_header.numBlocks[BLOCK19]);

	fseek(fh, m_header.blockOffset[BLOCK19], SEEK_SET);
	fread(&m_Block19[0], sizeof(PMMBlock19), m_header.numBlocks[BLOCK19], fh);

	for (int x=0; x<m_Block19.size(); x++)
	{
		byteSwapF32(m_Block19[x].u0x00);
		byteSwapF32(m_Block19[x].u0x04);
		byteSwapF32(m_Block19[x].u0x08);
		byteSwapF32(m_Block19[x].u0x0C);
		byteSwapF32(m_Block19[x].u0x10);
		byteSwapF32(m_Block19[x].u0x14);
	}
}

void CPMModel::ReadBlock20 (FILE* fh)
{
	m_Block20.resize(m_header.numBlocks[BLOCK20]);

	fseek(fh, m_header.blockOffset[BLOCK20], SEEK_SET);
	fread(&m_Block20[0], sizeof(PMMBlock20), m_header.numBlocks[BLOCK20], fh);

	for (int x=0; x<m_Block20.size(); x++)
	{
		byteSwapS32(m_Block20[x].idxBlock21);
		byteSwapU32(m_Block20[x].u0x04);
	}
}

void CPMModel::ReadBlock21 (FILE* fh)
{
	m_Block21.resize(m_header.numBlocks[BLOCK21]);

	fseek(fh, m_header.blockOffset[BLOCK21], SEEK_SET);
	fread(&m_Block21[0], sizeof(PMMBlock21), m_header.numBlocks[BLOCK21], fh);

	for (int x=0; x<m_Block21.size(); x++)
	{
		byteSwapU32(m_Block21[x].u0x00);
		byteSwapS32(m_Block21[x].idxTex);
		byteSwapU32(m_Block21[x].u0x08);
	}
}

void CPMModel::ReadBlock22 (FILE* fh)
{
	m_Block22.resize(m_header.numBlocks[BLOCK22]);

	fseek(fh, m_header.blockOffset[BLOCK22], SEEK_SET);
	fread(&m_Block22[0], sizeof(PMMBlock22), m_header.numBlocks[BLOCK22], fh);

	for (int x=0; x<m_Block22.size(); x++)
	{
		byteSwapU32(m_Block22[x].u0x00);
		byteSwapU32(m_Block22[x].u0x04);
		byteSwapU32(m_Block22[x].u0x08);
		byteSwapU32(m_Block22[x].u0x0C);
		byteSwapS32(m_Block22[x].idxBlock20);
		byteSwapU32(m_Block22[x].u0x14);
		byteSwapU32(m_Block22[x].u0x18);
		byteSwapU32(m_Block22[x].u0x1C);
		byteSwapU32(m_Block22[x].u0x20);
		byteSwapU32(m_Block22[x].u0x24);
		byteSwapU32(m_Block22[x].u0x28);
		byteSwapU32(m_Block22[x].u0x2C);
		byteSwapU32(m_Block22[x].u0x30);
		byteSwapU32(m_Block22[x].u0x34);
		byteSwapS32(m_Block22[x].idxBlock3);
		byteSwapS32(m_Block22[x].numBlock3);
		byteSwapS32(m_Block22[x].idxBlock5);
		byteSwapS32(m_Block22[x].idxBlock7);
		byteSwapS32(m_Block22[x].idxBlock9);
		byteSwapS32(m_Block22[x].idxBlock10);
		byteSwapU32(m_Block22[x].u0x50);
		byteSwapU32(m_Block22[x].u0x54);
		byteSwapU32(m_Block22[x].u0x58);
		byteSwapU32(m_Block22[x].u0x5C);
		byteSwapU32(m_Block22[x].u0x60);
		byteSwapU32(m_Block22[x].u0x64);
		byteSwapU32(m_Block22[x].u0x68);
	}
}

void CPMModel::ReadBlock23 (FILE* fh)
{
	m_Block23.resize(m_header.numBlocks[BLOCK23]);

	fseek(fh, m_header.blockOffset[BLOCK23], SEEK_SET);
	fread(&m_Block23[0], sizeof(PMMBlock23), m_header.numBlocks[BLOCK23], fh);
}

void CPMModel::ReadBlock24 (FILE* fh)
{
	m_Block24.resize(m_header.numBlocks[BLOCK24]);

	fseek(fh, m_header.blockOffset[BLOCK24], SEEK_SET);
	fread(&m_Block24[0], sizeof(PMMBlock24), m_header.numBlocks[BLOCK24], fh);

	for (int x=0; x<m_Block24.size(); x++)
		byteSwapF32(m_Block24[x].transform);
}

void CPMModel::ReadBlock25 (FILE* fh)
{
	m_Block25.resize(m_header.numBlocks[BLOCK25]);

	fseek(fh, m_header.blockOffset[BLOCK25], SEEK_SET);
	fread(&m_Block25[0], sizeof(PMMBlock25), m_header.numBlocks[BLOCK25], fh);

	for (int x=0; x<m_Block25.size(); x++)
	{
		byteSwapS32(m_Block25[x].nextRecord);
		byteSwapS32(m_Block25[x].childRecord);
		byteSwapS32(m_Block25[x].idxBlock2);
		byteSwapS32(m_Block25[x].idxBlock23);
		byteSwapS32(m_Block25[x].idxBlock24);
		byteSwapS32(m_Block25[x].joint);
	}
}

void CPMModel::ReadBlock26 (FILE* fh)
{
	m_Block26.resize(m_header.numBlocks[BLOCK26]);

	fseek(fh, m_header.blockOffset[BLOCK26], SEEK_SET);
	fread(&m_Block26[0], sizeof(PMMBlock26), m_header.numBlocks[BLOCK26], fh);

	for (int x=0; x<m_Block26.size(); x++)
	{
		byteSwapU32(m_Block26[x].u0x34);
		byteSwapU32(m_Block26[x].u0x38);
		byteSwapU32(m_Block26[x].addrBlock27);
	}
}

void CPMModel::ReadBlock27 (FILE* fh)
{
	m_Block27.resize(m_header.numBlocks[BLOCK26]);

	for (int x=0; x<m_Block26.size(); x++)
	{
		fseek(fh, m_Block26[x].addrBlock27, SEEK_SET);
		ReadBlock27_1(fh, x);
		ReadBlock27_2(fh, x);
		ReadBlock27_3(fh, x);
		ReadBlock27_4(fh, x);
		ReadBlock27_5(fh, x);
		ReadBlock27_6(fh, x);
		ReadBlock27_7(fh, x);
		ReadBlock27_8(fh, x);
		ReadBlock27_9(fh, x);
	}
}

void CPMModel::ReadBlock27_1 (FILE* fh, int idx)
{
	fread(&m_Block27[idx].header, sizeof(PMMBlock27_Header), 1, fh);

	byteSwapU32(m_Block27[idx].header.size);
	byteSwapF32(m_Block27[idx].header.u0x44);
	byteSwapF32(m_Block27[idx].header.u0x48);
	byteSwapF32(m_Block27[idx].header.u0x4C);
	byteSwapF32(m_Block27[idx].header.u0x50);
	byteSwapF32(m_Block27[idx].header.u0x54);
	byteSwapF32(m_Block27[idx].header.u0x58);

	for (int x=0; x<8; x++)
		byteSwapU32(m_Block27[idx].header.numBlocks[x]);

	for (int x=0; x<8; x++)
		byteSwapU32(m_Block27[idx].header.blockOffset[x]);
}

void CPMModel::ReadBlock27_2 (FILE* fh, int idx)
{
	m_Block27[idx].Block2.resize(m_Block27[idx].header.numBlocks[BLOCK27_2]);

	fseek(fh, m_Block26[idx].addrBlock27 + m_Block27[idx].header.blockOffset[BLOCK27_2], SEEK_SET);
	fread(&m_Block27[idx].Block2[0], sizeof(PMMBlock27_2), m_Block27[idx].header.numBlocks[BLOCK27_2], fh);
	
	for (int x=0; x<m_Block27[idx].Block2.size(); x++)
	{
		byteSwapU32(m_Block27[idx].Block2[x].u0x00);
		byteSwapU32(m_Block27[idx].Block2[x].u0x04);
		byteSwapF32(m_Block27[idx].Block2[x].time);
	}
}

void CPMModel::ReadBlock27_3 (FILE* fh, int idx)
{
	m_Block27[idx].Block3.resize(m_Block27[idx].header.numBlocks[BLOCK27_3]);

	fseek(fh, m_Block26[idx].addrBlock27 + m_Block27[idx].header.blockOffset[BLOCK27_3], SEEK_SET);
	fread(&m_Block27[idx].Block3[0], sizeof(PMMBlock27_3), m_Block27[idx].header.numBlocks[BLOCK27_3], fh);

	for (int x=0; x<m_Block27[idx].Block3.size(); x++)
	{
		byteSwapF32(m_Block27[idx].Block3[x].time);
		byteSwapU32(m_Block27[idx].Block3[x].idxBlock4);
		byteSwapU32(m_Block27[idx].Block3[x].numBlock4);
		byteSwapU32(m_Block27[idx].Block3[x].idxBlock5);
		byteSwapU32(m_Block27[idx].Block3[x].numBlock5);
		byteSwapU32(m_Block27[idx].Block3[x].idxBlock6);
		byteSwapU32(m_Block27[idx].Block3[x].numBlock6);
		byteSwapU32(m_Block27[idx].Block3[x].idxBlock7);
		byteSwapU32(m_Block27[idx].Block3[x].numBlock7);
		byteSwapU32(m_Block27[idx].Block3[x].idxBlock8);
		byteSwapU32(m_Block27[idx].Block3[x].numBlock8);
	}
}

void CPMModel::ReadBlock27_4 (FILE* fh, int idx)
{
	m_Block27[idx].Block4.resize(m_Block27[idx].header.numBlocks[BLOCK27_4]);

	fseek(fh, m_Block26[idx].addrBlock27 + m_Block27[idx].header.blockOffset[BLOCK27_4], SEEK_SET);
	fread(&m_Block27[idx].Block4[0], sizeof(PMMBlock27_4), m_Block27[idx].header.numBlocks[BLOCK27_4], fh);
	
	for (int x=0; x<m_Block27[idx].Block4.size(); x++)
	{
		byteSwapU32(m_Block27[idx].Block4[x].u0x00);
	}
}

void CPMModel::ReadBlock27_5 (FILE* fh, int idx)
{
	m_Block27[idx].Block5.resize(m_Block27[idx].header.numBlocks[BLOCK27_5]);

	fseek(fh, m_Block26[idx].addrBlock27 + m_Block27[idx].header.blockOffset[BLOCK27_5], SEEK_SET);
	fread(&m_Block27[idx].Block5[0], sizeof(PMMBlock27_5), m_Block27[idx].header.numBlocks[BLOCK27_5], fh);
	
	for (int x=0; x<m_Block27[idx].Block5.size(); x++)
	{
		byteSwapU32(m_Block27[idx].Block5[x].u0x00);
	}
}

void CPMModel::ReadBlock27_6 (FILE* fh, int idx)
{
	m_Block27[idx].Block6.resize(m_Block27[idx].header.numBlocks[BLOCK27_6]);

	fseek(fh, m_Block26[idx].addrBlock27 + m_Block27[idx].header.blockOffset[BLOCK27_6], SEEK_SET);
	fread(&m_Block27[idx].Block6[0], sizeof(PMMBlock27_6), m_Block27[idx].header.numBlocks[BLOCK27_6], fh);
	
	for (int x=0; x<m_Block27[idx].Block6.size(); x++)
	{
		byteSwapU32(m_Block27[idx].Block6[x].u0x00);
		byteSwapU32(m_Block27[idx].Block6[x].u0x04);
		byteSwapU32(m_Block27[idx].Block6[x].u0x08);
	}
}

void CPMModel::ReadBlock27_7 (FILE* fh, int idx)
{
	m_Block27[idx].Block7.resize(m_Block27[idx].header.numBlocks[BLOCK27_7]);

	fseek(fh, m_Block26[idx].addrBlock27 + m_Block27[idx].header.blockOffset[BLOCK27_7], SEEK_SET);
	fread(&m_Block27[idx].Block7[0], sizeof(PMMBlock27_7), m_Block27[idx].header.numBlocks[BLOCK27_7], fh);
}

void CPMModel::ReadBlock27_8 (FILE* fh, int idx)
{
	m_Block27[idx].Block8.resize(m_Block27[idx].header.numBlocks[BLOCK27_8]);

	fseek(fh, m_Block26[idx].addrBlock27 + m_Block27[idx].header.blockOffset[BLOCK27_8], SEEK_SET);
	fread(&m_Block27[idx].Block8[0], sizeof(PMMBlock27_8), m_Block27[idx].header.numBlocks[BLOCK27_8], fh);
	
	for (int x=0; x<m_Block27[idx].Block8.size(); x++)
	{
		byteSwapU32(m_Block27[idx].Block8[x].u0x00);
	}
}

void CPMModel::ReadBlock27_9 (FILE* fh, int idx)
{
	m_Block27[idx].Block9.resize(m_Block27[idx].header.numBlocks[BLOCK27_9]);

	fseek(fh, m_Block26[idx].addrBlock27 + m_Block27[idx].header.blockOffset[BLOCK27_9], SEEK_SET);
	fread(&m_Block27[idx].Block9[0], sizeof(PMMBlock27_9), m_Block27[idx].header.numBlocks[BLOCK27_9], fh);
	
	for (int x=0; x<m_Block27[idx].Block9.size(); x++)
	{
		byteSwapU32(m_Block27[idx].Block9[x].u0x00);
		byteSwapU32(m_Block27[idx].Block9[x].u0x04);
	}
}