#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glaux.h>
#include <vector>
#include <iostream>
#include "pmmodelgl.h"
#include "pmmodel.h"
#include "common.h"
#include "tpl.h"
#include "color.h"

CPMModelGL::CPMModelGL ()
{
	m_showTex = 1;
}

CPMModelGL::~CPMModelGL ()
{
}

bool CPMModelGL::LoadFile (const string &filename)
{
	if (!CPMModel::LoadFile(filename))
		return false;
	
	//m_tpl = new TPL();
	if (!m_tpl.LoadFile(m_header.textureFile))
		return false;

	LoadTextures();

	return true;
}

bool CPMModelGL::LoadTextures ()
{
	int ntex = m_tpl.numTextures();

	m_gltex.resize(ntex);

	for (int x=0; x<ntex; x++)
	{
		vector<Color> data;
		vector<Color> sdata;
		int width = m_tpl.m_tex[x].texHeader->width;
		int height = m_tpl.m_tex[x].texHeader->height;
		int swidth = 64;
		int sheight = 64;
		for (int mw=64; mw<width; mw*=2)
			swidth = mw*=2;
		for (int mh=64; mh<height; mh*=2)
			sheight = mh*=2;
		sdata.resize(swidth*sheight);
		data = m_tpl.DecodeTexture(x);

		/*for (int a=0, c=0; a<sheight; a++) {
			if (height <= a)
				break;
			for (int b=0; b<swidth; b++) {
				if (width <= b)
					break;
				sdata[a*swidth+b] = data[c];
				c++;
			}
		}*/

		//---
		gluScaleImage(GL_RGBA, width, height, GL_UNSIGNED_BYTE, &data[0], swidth, sheight, GL_UNSIGNED_BYTE, &sdata[0]);
		
		//---
		glGenTextures(1, &m_gltex[x]);
		glBindTexture(GL_TEXTURE_2D, m_gltex[x]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, swidth, sheight, 0, GL_RGBA, GL_UNSIGNED_BYTE, &sdata[0]);

	}

	GLubyte blank[4] = {255,255,255,255};
	glGenTextures(1, &m_blank);
	glBindTexture(GL_TEXTURE_2D, m_blank);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 1,1, 0, GL_RGBA, GL_UNSIGNED_BYTE, blank);

	return true;
}

void CPMModelGL::TransformNode (s32 idxB25)
{
	PMMBlock25 cB25 = m_Block25[idxB25];

	glPushMatrix();
	//1 7 2 8 4 5 3 6

	// 1: Translate
	glTranslatef(m_Block24[cB25.idxBlock24+ 0].transform,
				 m_Block24[cB25.idxBlock24+ 1].transform,
				 m_Block24[cB25.idxBlock24+ 2].transform);

	// 7: Scale Translation
	glTranslatef(m_Block24[cB25.idxBlock24+18].transform,
				 m_Block24[cB25.idxBlock24+19].transform,
				 m_Block24[cB25.idxBlock24+20].transform);

	// 2: Scale
	glScalef(m_Block24[cB25.idxBlock24+ 3].transform,
			 m_Block24[cB25.idxBlock24+ 4].transform,
			 m_Block24[cB25.idxBlock24+ 5].transform);

	// 8: Scale Counter-Translation
	glTranslatef(0-m_Block24[cB25.idxBlock24+21].transform,
				 0-m_Block24[cB25.idxBlock24+22].transform,
				 0-m_Block24[cB25.idxBlock24+23].transform);

	// 4: Rotate B
	glRotatef(m_Block24[cB25.idxBlock24+11].transform, 0.0f, 0.0f, 1.0f);
	glRotatef(m_Block24[cB25.idxBlock24+10].transform, 0.0f, 1.0f, 0.0f);
	glRotatef(m_Block24[cB25.idxBlock24+ 9].transform, 1.0f, 0.0f, 0.0f);

	// 5: Rotate Translation
	glTranslatef(m_Block24[cB25.idxBlock24+12].transform,
				 m_Block24[cB25.idxBlock24+13].transform,
				 m_Block24[cB25.idxBlock24+14].transform);

	// 3: Rotate A
	glRotatef(m_Block24[cB25.idxBlock24+ 8].transform*2, 0.0f, 0.0f, 1.0f);
	glRotatef(m_Block24[cB25.idxBlock24+ 7].transform*2, 0.0f, 1.0f, 0.0f);
	glRotatef(m_Block24[cB25.idxBlock24+ 6].transform*2, 1.0f, 0.0f, 0.0f);

	// 6: Rotate Counter-Translation
	glTranslatef(0-m_Block24[cB25.idxBlock24+15].transform,
				 0-m_Block24[cB25.idxBlock24+16].transform,
				 0-m_Block24[cB25.idxBlock24+17].transform);

	// Process Children
	if (cB25.childRecord != -1)
		TransformNode(cB25.childRecord);

	// Draw Node
	DrawNode(idxB25);

	glPopMatrix();

	// Process Siblings
	if (cB25.nextRecord != -1)
		TransformNode(cB25.nextRecord);
}

void CPMModelGL::DrawNode (s32 idxB25)
{
	PMMBlock25 cBlock25 = m_Block25[idxB25];

	if (cBlock25.idxBlock2 != -1)
	{
		PMMBlock2 cBlock2 = m_Block2[cBlock25.idxBlock2];
		PMMBlock23 cBlock23 = m_Block23[cBlock25.idxBlock23];

		if (cBlock23.control == 0)
			return;

		// Blending
		if (cBlock2.u0xA0 == 0) {
			glAlphaFunc(GL_GREATER, 0.95f);
			glEnable(GL_ALPHA_TEST);
			glDisable(GL_BLEND);
		} else if (cBlock2.u0xA0 == 3) {
			glBlendFunc(GL_SRC_ALPHA, GL_ONE);
			glEnable(GL_BLEND);
			glDisable(GL_ALPHA_TEST);
		}

		// Culling
		if (cBlock2.u0xA4 == 1)
			glEnable(GL_CULL_FACE);
		if (cBlock2.u0xA4 == 3)
			glDisable(GL_CULL_FACE);

		for (int b=0; b<cBlock2.numBlock22; b++)
		{
			PMMBlock22 cBlock22 = m_Block22[cBlock2.idxBlock22+b];
			//if (cBlock22.u0x04 == 0)
			//	continue;

			for (int c=0; c<cBlock22.numBlock3; c++)
			{
				PMMBlock3 cBlock3 = m_Block3[cBlock22.idxBlock3+c];

				if (m_showTex && m_Block21.size() > 0 && cBlock22.idxBlock20 != -1) {
					glBindTexture(GL_TEXTURE_2D, m_gltex[m_Block20[cBlock22.idxBlock20].idxBlock21]);
				} else {
					glDisable(GL_TEXTURE_2D);
				}

				switch(cBlock3.vertices)
				{
					case 4: glBegin(GL_QUADS); break;
					case 3: glBegin(GL_TRIANGLES); break;
					case 2: glBegin(GL_LINES); break;
					case 1: glBegin(GL_POINTS); break;
					default: glBegin(GL_POLYGON); break;
				}

				for (int d=0; d<cBlock3.vertices; d++)
				{
					int ppoint = m_Block5[cBlock22.idxBlock5+cBlock3.extention+d].vertex;
					PMMBlock4 vertex = m_Block4[cBlock2.idxBlock4+ppoint];
					PMMBlock6 normal = m_Block6[m_Block7[cBlock22.idxBlock7+cBlock3.extention+d].idxBlock6];
					PMMBlock8 color	= m_Block8[m_Block9[cBlock22.idxBlock9+cBlock3.extention+d].idxBlock8];

					glColor4ub(color.r, color.g, color.b, color.a);
					glNormal3f(normal.nx, normal.ny, normal.nz);

					if (m_showTex && m_Block21.size() > 0 && cBlock22.idxBlock20 != -1) {
						int texCoordVtx = m_Block10[cBlock22.idxBlock10+cBlock3.extention+d].vertex;
						PMMBlock18 texcoord = m_Block18[cBlock2.idxBlock18+texCoordVtx];
						glTexCoord2f(texcoord.s, texcoord.t);
					}

					f32 vx = vertex.x;
					f32 vy = vertex.y;
					f32 vz = vertex.z;
					glVertex3f(vx, vy, vz);
				}

				glEnd();
			} // d
		} // c
	} // b
}

void CPMModelGL::DrawBox (f32 p0x, f32 p0y, f32 p0z, f32 p1x, f32 p1y, f32 p1z)
{
	glDisable(GL_CULL_FACE);
	glDisable(GL_TEXTURE_2D);
	glColor3f(1.0f, 1.0f, 0.0f);
	glBegin(GL_LINES);
		glVertex3f(p0x, p0y, p0z);
		glVertex3f(p1x, p0y, p0z);
		glVertex3f(p1x, p0y, p0z);
		glVertex3f(p1x, p0y, p1z);
		glVertex3f(p1x, p0y, p1z);
		glVertex3f(p0x, p0y, p1z);
		glVertex3f(p0x, p0y, p1z);
		glVertex3f(p0x, p0y, p0z);

		glVertex3f(p0x, p1y, p0z);
		glVertex3f(p1x, p1y, p0z);
		glVertex3f(p1x, p1y, p0z);
		glVertex3f(p1x, p1y, p1z);
		glVertex3f(p1x, p1y, p1z);
		glVertex3f(p0x, p1y, p1z);
		glVertex3f(p0x, p1y, p1z);
		glVertex3f(p0x, p1y, p0z);

		glVertex3f(p0x, p0y, p0z);
		glVertex3f(p0x, p1y, p0z);
		glVertex3f(p1x, p0y, p0z);
		glVertex3f(p1x, p1y, p0z);
		glVertex3f(p1x, p0y, p1z);
		glVertex3f(p1x, p1y, p1z);
		glVertex3f(p0x, p0y, p1z);
		glVertex3f(p0x, p1y, p1z);
	glEnd();
}

void CPMModelGL::DrawBoundingBox ()
{
	DrawBox(m_header.u0xD0, m_header.u0xD4, m_header.u0xD8, m_header.u0xDC, m_header.u0xE0, m_header.u0xE4);
}

void CPMModelGL::showTextures (u32 set)
{
	m_showTex = set;
}