#ifndef TPL_H
#define TPL_H

#include <windows.h>
#include <gl\gl.h>
#include <vector>
#include "common.h"
#include "color.h"

using namespace std;

struct TPLHeader {
	u32		magic;
	u32		ntextures;
	u32		sizeHeader;
};

struct TPLTexture {
	u32		texHeaderOffset;
	u32		texPalOffset;
};

struct TPLTexHeader {
	u16		height;
	u16		width;
	u32		format;
	u32		texDataOffset;
	u32		wrap_s;
	u32		wrap_t;
	u32		min_filter;
	u32		mag_filter;
	f32		lod_bias;
	u8		edge_lod;
	u8		min_lod;
	u8		max_lod;
	u8		unpacked;
};

struct TPLPalHeader {
	u16		nitems;
	u8		unpacked;
	u8		pad;
	u32		format;
	u32		palDataOffset;
};

struct TPLItem {
	TPLTexHeader*	texHeader;
	TPLPalHeader*	palHeader;
	vector<u8>		data;
};

class TPL
{
public:
	TPL ();
	~TPL ();

	bool LoadFile (const string &filename);
	
	vector<Color> DecodeTexture (u32 index);
	vector<Color> DecodeStandard (u32 index, u32 type);
	vector<Color> DecodeS3TC (u32 index);

	inline int numTextures () { return m_header.ntextures; }

	vector<TPLItem>	m_tex;

private:
	string		m_filename;
	TPLHeader	m_header;

	u32	getTileWidth (u32 type);
	u32 getTileHeight (u32 type);
	f32	getBytePP (u32 type);

	vector<TPLTexture>		m_textures;
	vector<TPLTexHeader>	m_texHeaders;
	vector<TPLPalHeader>	m_palHeaders;
};

#endif // TPL_H